//
//  XSPEC12  November 2003
//
//

#include <xsTypes.h>
#include <XSUtil/FunctionUtils/funcType.h>
#include <cmath>
#include <iomanip>

#include <PsfTraits.h>
#include <Psf.h>
#include <GlobalContainer/ModelContainer.h>

extern "C" void suzaku (const EnergyPointer& energyArray,
                                  const std::vector<Real>& parameterValues,
                                  GroupFluxContainer& flux,
                                  GroupFluxContainer& fluxError,
                                  MixBase** mixGenerator,
                                  const string& modName)
{
   using namespace XSContainer;
   static const string SUZAKU("Suzaku");
   // mixBase pointers are owned by ModelContainer which is a singleton.

   ModelContainer* m(ModelContainer::Instance());
   try
   {
      if (!*mixGenerator)
      {

            if (!m->mixingComponents(SUZAKU))
            {
                      m->mixingComponents(SUZAKU,new Psf<Suzaku>(SUZAKU));   
            }
            (*mixGenerator) = m->mixingComponents(SUZAKU);
            (*mixGenerator)->initialize(parameterValues);
      }
      // Refresh is needed only if parameters are variable and
      // there is no image file.  This is dealt with inside the
      // following function.
      (*mixGenerator)->refreshTransform();
      (*mixGenerator)->perform(energyArray,parameterValues,flux,fluxError);
   }
   catch (...)
   {
      // If it throws out of here, MixComponent::perform may never
      // get the chance to set its mixGenerator pointer.  Therefore,
      // can't rely on it to initiate destruction of the faulty
      // MixBase object.  So, do it now.
      m->removeMixingComponent(SUZAKU);
      *mixGenerator = 0;
      throw;
   }
}
